import { DI } from '@microsoft/fast-foundation';
/**
 * Default TypedBroadcastChannel implementation.
 * @public
 */
export class DefaultTypedBroadcastChannel extends BroadcastChannel {
    constructor() {
        super(...arguments);
        /** {@inheritDoc TypedBroadcastChannel.postMessage} */
        // @ts-ignore
        this.postMessage = (...args) => {
            const [type, detail] = args;
            super.postMessage({
                type,
                detail,
            });
        };
        /** {@inheritDoc TypedBroadcastChannel.isMessageType} */
        this.isMessageType = (type, event) => {
            var _a;
            return ((_a = event.data) === null || _a === void 0 ? void 0 : _a.type) === type;
        };
    }
}
/**
 * Creates a dependency injection key for the broadcast channel being registered which is lazily created when injected.
 * @example In a package's config.ts
 * ```ts
 * export interface GenesisResourcesChannel extends TypedBroadcastChannel<GenesisResourcesEvents> {}
 * ...
 * export const GenesisResourcesChannel = registerTypedBroadcastChannel<GenesisResourcesEvents>(GenesisResourcesChannelId);
 * ```
 * @param name - The channel name.
 * @returns The created key.
 * @public
 */
export const registerTypedBroadcastChannel = (name) => {
    class Value extends DefaultTypedBroadcastChannel {
        constructor() {
            super(name);
        }
    }
    return DI.createInterface((x) => x.transient(Value));
};
/**
 * Creates a strongly typed broadcast channel.
 *
 * @remarks
 * `registerTypedBroadcastChannel` may be preferred, as it will create and register the channel in the DI.
 *
 * @example
 * ```ts
 * const channel = createTypedBroadcastChannel<GenesisResourcesEvents>(GenesisResourcesChannelId);
 * ```
 * @param name - The channel name.
 * @returns The created TypedBroadcastChannel.
 * @public
 */
export const createTypedBroadcastChannel = (name) => {
    return new DefaultTypedBroadcastChannel(name);
};
